/*
'  DotNetNuke -  http://www.dotnetnuke.com
'  Copyright (c) 2002-2012
'  by DotNetNuke Corporation
' 
'  Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
'  documentation files (the "Software"), to deal in the Software without restriction, including without limitation 
'  the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and 
'  to permit persons to whom the Software is furnished to do so, subject to the following conditions:
' 
'  The above copyright notice and this permission notice shall be included in all copies or substantial portions 
'  of the Software.
' 
'  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED 
'  TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
'  THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
'  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
'  DEALINGS IN THE SOFTWARE.
*/

using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Reflection;
using DotNetNuke.Common;
using DotNetNuke.Common.Utilities;

namespace DotNetNuke.Modules.Store.Components
{
	/// <summary>
	/// Abtract class used to manage querystring parameters.
	/// </summary>
	public abstract class NavigateWrapper
	{
		#region Private Members

		private int _tabID = Null.NullInteger;
		private int _moduleID = Null.NullInteger;
		private string _controlKey = Null.NullString;

		#endregion

		#region Constructors

	    protected NavigateWrapper()
		{
		}

	    protected NavigateWrapper(NameValueCollection queryString)
		{
			LoadQueryString(queryString);
		}

	    protected NavigateWrapper(int moduleId, NameValueCollection queryString)
		{
            _moduleID = moduleId;
			LoadQueryString(queryString);
		}

		#endregion

		#region Properties

		public int TabID
		{
			get { return _tabID; }
			set { _tabID = value; }
		}

		public int ModuleID
		{
            get { return _moduleID; }
            set { _moduleID = value; }
		}

		public string ControlKey
		{
			get { return _controlKey; }
			set { _controlKey = value; }
		}

		#endregion

		#region Public Methods

		/// <summary>
		/// Parses the queryString parameters and sets properties, if they exist,
		/// in the derived object.
		/// </summary>
		/// <param name="queryString">Querystring</param>
		public void LoadQueryString(NameValueCollection queryString)
		{
            PropertyInfo[] propertyList = GetProperties();
            //string queryValue = null;
		    // Iterate thru all properties for this type
			foreach(PropertyInfo property in propertyList)
			{
			    // Do we have a value for this property?
			    object value = queryString[property.Name];
			    if (value != null)
                    property.SetValue(this, Convert.ChangeType(value, property.PropertyType), null);
			}
		}

		/// <summary>
		/// Gets an array of querystring parameters to be used with the 
		/// DotNetNuke NavigateUrl(), EditUrl() etc.
		/// </summary>
        /// <returns>Querystring parameters</returns>
		public string[] GetNavigateParameters()
		{
			return GetNavigateParameters(new StringDictionary());
		}

		/// <summary>
		/// Gets an array of querystring parameters to be used with the 
		/// DotNetNuke NavigateUrl(), EditUrl() etc.
		/// </summary>
		/// <param name="replaceParams">List of parameters to use as replacements for existing parameters.</param>
        /// <returns>Querystring parameters</returns>
		public string[] GetNavigateParameters(StringDictionary replaceParams)
		{
			PropertyInfo[] propertyList = GetProperties();
            List<string> settingList = new List<string>(propertyList.Length);
		    object propertyValue;
			// Iterate thru all properties for this type
			foreach(PropertyInfo property in propertyList)
			{
			    string propertyName = property.Name;
			    if (replaceParams.ContainsKey(propertyName))
			    {
			        string replaceValue = replaceParams[propertyName];
			        if (string.IsNullOrEmpty(replaceValue) == false)
                    {
                        // Add name and value pair for NavigateUrl() parameters
                        if (propertyName == "ModuleID")
                            settingList.Add("mid=" + replaceParams[propertyName]);
                        else
                            settingList.Add(propertyName + "=" + replaceParams[propertyName]);
                    }
			    }
			    else
				{
                    if (propertyName != "TabID" && propertyName != "ControlKey")
                    {
                        propertyValue = property.GetValue(this, null);
                        if (Null.IsNull(propertyValue) == false)
                        {
                            // Add name and value pair for NavigateUrl() parameters
                            if (propertyName == "ModuleID")
                                settingList.Add("mid=" + propertyValue);
                            else
                                settingList.Add(propertyName + "=" + propertyValue);
                        }
                    }
				}
			}
		    // Cast to string array and return
			return (settingList.ToArray());			
		}

		/// <summary>
		/// Gets the url for navigation including the appropriate parameters.
		/// </summary>
		/// <returns>URL</returns>
		public string GetNavigationUrl()
		{
			return Globals.NavigateURL(_tabID, _controlKey, GetNavigateParameters()).ToLower();
		}

		/// <summary>
		/// Gets the url for navigation including the appropriate parameters.
		/// </summary>
		/// <param name="replaceParams">List of parameters to use as replacements for existing parameters.</param>
		/// <returns>URL</returns>
		public string GetNavigationUrl(StringDictionary replaceParams)
		{
			return Globals.NavigateURL(_tabID, _controlKey, GetNavigateParameters(replaceParams)).ToLower();
		}

	    /// <summary>
	    /// Gets the url for navigation including the appropriate parameters.
	    /// </summary>
	    /// <param name="tabID">Current tab ID.</param>
	    /// <param name="replaceParams">List of parameters to use as replacements for existing parameters.</param>
	    /// <returns>URL</returns>
	    public string GetNavigationUrl(int tabID, StringDictionary replaceParams)
        {
            return Globals.NavigateURL(tabID, _controlKey, GetNavigateParameters(replaceParams)).ToLower();
        }

        public string GetEditUrl()
        {
            return GetEditUrl("", "", _controlKey);
        }

        public string GetEditUrl(int moduleID)
        {
            return GetEditUrl(moduleID, "", "", _controlKey);
        }

        public string GetEditUrl(string controlKey)
        {
            return GetEditUrl("", "", controlKey);
        }

        public string GetEditUrl(int moduleID, string controlKey)
        {
            return GetEditUrl(moduleID, "", "", controlKey);
        }

        public string GetEditUrl(string keyName, string keyValue)
        {
            return GetEditUrl(keyName, keyValue, _controlKey);
        }

        public string GetEditUrl(int moduleID, string keyName, string keyValue)
        {
            return GetEditUrl(moduleID, keyName, keyValue, _controlKey);
        }

        public string GetEditUrl(string keyName, string keyValue, string controlKey)
        {
            if (_moduleID == Null.NullInteger)
                throw new Exception("GetEditUrl: ModuleID is required!");

            StringDictionary replaceParams = new StringDictionary();
            replaceParams.Add("moduleid", _moduleID.ToString());

            if (keyName != "" && keyValue != "")
                replaceParams.Add(keyName, keyValue);

            if (string.IsNullOrEmpty(controlKey))
                controlKey = "Edit";

            return Globals.NavigateURL(_tabID, controlKey, GetNavigateParameters(replaceParams));
        }

        public string GetEditUrl(int moduleID, string keyName, string keyValue, string controlKey)
        {
            _moduleID = moduleID;
            return GetEditUrl(keyName, keyValue, controlKey);
        }

		#endregion

        #region Private Methods

        private PropertyInfo[] GetProperties()
        {
            string cacheKey = "Store_" + GetType().FullName;
            PropertyInfo[] propertyList = (PropertyInfo[])DataCache.GetCache(cacheKey);
            if (propertyList == null)
            {
                propertyList = GetType().GetProperties();
                DataCache.SetCache(cacheKey, propertyList);
            }
            return propertyList;
        }

        #endregion
    }
}
